"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _forkedRebound = require("../forked-rebound");

var _utilities = require("../utilities");

var _springSettings = require("../springSettings");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// this should get created only 1x
var springSystem = new _forkedRebound.SpringSystem();
/**
 * A simple spring function for animating DOM properties.
 * Returns a function that will immediately cancel the in-progress animation.
 * */

var createSimpleSpring = function createSimpleSpring(_ref) {
  var config = _ref.config,
      values = _ref.values,
      onUpdate = _ref.onUpdate,
      _ref$delay = _ref.delay,
      delay = _ref$delay === void 0 ? 0 : _ref$delay,
      onComplete = _ref.onComplete;

  var _ref2 = (0, _utilities.assign)({}, _springSettings.springPresets.noWobble, (0, _springSettings.normalizeSpring)(config)),
      stiffness = _ref2.stiffness,
      damping = _ref2.damping,
      overshootClamping = _ref2.overshootClamping;

  var spring = springSystem.createSpring(stiffness, damping);
  spring.setOvershootClampingEnabled(!!overshootClamping);
  spring.addListener({
    onSpringAtRest: function onSpringAtRest(spring) {
      if (onComplete) onComplete();
      spring.destroy();
    },
    onSpringUpdate: function onSpringUpdate(spring) {
      var springVal = spring.getCurrentValue();
      if (!values) return onUpdate(springVal);
      var currentValues = Object.keys(values).map(function (value) {
        return [value, (0, _utilities.tweenProp)(values[value][0], values[value][1], springVal)];
      }).reduce(function (acc, curr) {
        return Object.assign(acc, _defineProperty({}, curr[0], curr[1]));
      }, {});
      onUpdate(currentValues);
    }
  });

  if (delay) {
    setTimeout(function () {
      spring.setEndValue(1);
    }, delay);
  } else {
    spring.setEndValue(1);
  }

  return spring;
};

var _default = createSimpleSpring;
exports.default = _default;