"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _onFrame = _interopRequireDefault(require("../forked-rebound/onFrame"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var touchMove = 'touchmove';
var touchEnd = 'touchend';
var mouseMove = 'mousemove';
var mouseUp = 'mouseup';
var initialState = {
  event: undefined,
  target: undefined,
  time: undefined,
  xy: [0, 0],
  delta: [0, 0],
  initial: [0, 0],
  previous: [0, 0],
  local: [0, 0],
  lastLocal: [0, 0],
  velocity: 0,
  distance: 0,
  down: false,
  first: true
};
var defaultConfig = {
  window: window,
  touchOnly: true,
  onDown: undefined,
  onUp: undefined
};

function handlers(set, _ref) {
  var onAction = _ref.onAction,
      config = _ref.config;

  var handleUp = function handleUp(event) {
    set(function (state) {
      var newState = _objectSpread({}, state, {
        down: false,
        first: false // @ts-ignore

      });

      if (onAction) onAction(newState);
      if (config.onUp) config.onUp(newState);
      return _objectSpread({}, newState, {
        event: event,
        // @ts-ignore
        lastLocal: state.local
      });
    });
  };

  var handleDown = function handleDown(event) {
    var _ref2 = event.touches ? event.touches[0] : event,
        target = _ref2.target,
        pageX = _ref2.pageX,
        pageY = _ref2.pageY; // @ts-ignore


    set(function (state) {
      var lastLocal = state.lastLocal || initialState.lastLocal;

      var newState = _objectSpread({}, initialState, {
        event: event,
        target: target,
        lastLocal: lastLocal,
        local: lastLocal,
        xy: [pageX, pageY],
        initial: [pageX, pageY],
        previous: [pageX, pageY],
        down: true,
        time: Date.now(),
        cancel: function cancel() {
          stop(); // @ts-ignore

          (0, _onFrame.default)(function () {
            return handleUp(event);
          });
        }
      });

      onAction(newState); // @ts-ignore

      if (config.onDown) config.onDown(newState);
      return _objectSpread({}, newState);
    });
  };

  var handleMove = function handleMove(event) {
    var _ref3 = event.touches ? event.touches[0] : event,
        pageX = _ref3.pageX,
        pageY = _ref3.pageY; // @ts-ignore


    set(function (state) {
      var time = Date.now();
      var x_dist = pageX - state.xy[0];
      var y_dist = pageY - state.xy[1];
      var delta_x = pageX - state.initial[0];
      var delta_y = pageY - state.initial[1];
      var distance = Math.sqrt(delta_x * delta_x + delta_y * delta_y);
      var len = Math.sqrt(x_dist * x_dist + y_dist * y_dist);

      var newState = _objectSpread({}, state, {
        event: event,
        time: time,
        xy: [pageX, pageY],
        delta: [delta_x, delta_y],
        local: [state.lastLocal[0] + pageX - state.initial[0], state.lastLocal[1] + pageY - state.initial[1]],
        velocity: len / (time - state.time),
        distance: distance,
        previous: state.xy,
        first: false
      });

      onAction && onAction(newState);
      return _objectSpread({}, newState);
    });
  };

  var onDown = function onDown(event) {
    // it's a touch event with multiple fingers down but we only allow single finger swipes
    // @ts-ignore
    if (event.targetTouches && event.targetTouches.length > 1) return;
    if (config.maxWidth && !window.matchMedia("screen and (max-width: ".concat(config.maxWidth, "px)"))) return;

    if (!config.touchOnly) {
      config.window.addEventListener(mouseMove, handleMove, {
        passive: true
      }); // @ts-ignore

      config.window.addEventListener(mouseUp, onUp, {
        passive: true
      });
    }

    config.window.addEventListener(touchMove, handleMove, {
      passive: true
    }); // @ts-ignore

    config.window.addEventListener(touchEnd, onUp, {
      passive: true
    });
    handleDown(event);
  };

  var stop = function stop() {
    if (!config.touchOnly) {
      config.window.removeEventListener(mouseMove, handleMove, {
        // @ts-ignore
        passive: true
      }); // @ts-ignore

      config.window.removeEventListener(mouseUp, onUp, {
        passive: true
      });
    }

    config.window.removeEventListener(touchMove, handleMove, {
      // @ts-ignore
      passive: true
    }); // @ts-ignores

    config.window.removeEventListener(touchEnd, onUp, {
      passive: true
    });
  };

  var onUp = function onUp(e) {
    stop();
    handleUp(e);
  };

  var output = {};

  if (!config.touchOnly) {
    output["onMouseDown"] = onDown;
  }

  output["onTouchStart"] = onDown;
  return {
    handlers: output,
    set: set
  };
}

function gestureHandlers(props) {
  var _state = initialState; // @ts-ignore

  var set = function set(cb) {
    return _state = cb(_state);
  };

  return handlers(set, {
    onAction: props.onAction,
    // passing in an object so that we can automatically get updated settings without
    // rebinding the handlers
    config: Object.assign(defaultConfig, props.config)
  });
}

var _default = gestureHandlers;
exports.default = _default;