"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _gestureHandlers2 = _interopRequireDefault(require("./gestureHandlers"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultCompleteThreshhold = 0.25;

var getMovementScalar = function getMovementScalar(_ref) {
  var deltaX = _ref.deltaX,
      deltaY = _ref.deltaY,
      direction = _ref.direction;
  var normalized = {
    up: -deltaY,
    down: deltaY,
    left: -deltaX,
    right: deltaX
  };
  return normalized[direction];
};

var cleanUpAnimation = function cleanUpAnimation(inProgressAnimations, flipId) {
  if (inProgressAnimations[flipId] && inProgressAnimations[flipId].spring) {
    inProgressAnimations[flipId].spring.destroy();
  }

  delete inProgressAnimations[flipId];
};

var makeFinishFunction = function makeFinishFunction(endValue) {
  return function (_ref2) {
    var inProgressAnimations = _ref2.inProgressAnimations,
        onFinished = _ref2.onFinished,
        _ref2$velocity = _ref2.velocity,
        velocity = _ref2$velocity === void 0 ? 1 : _ref2$velocity;
    var springsCompleted = Object.keys(inProgressAnimations).map(function (flipId) {
      var _inProgressAnimations = inProgressAnimations[flipId],
          spring = _inProgressAnimations.spring,
          onAnimationEnd = _inProgressAnimations.onAnimationEnd;

      if (!spring) {
        return Promise.reject();
      }

      if (Object.keys(spring._cachedSpringConfig).length) {
        spring.setSpringConfig(spring._cachedSpringConfig);
      }

      spring.setVelocity(Math.max(Math.min(velocity, 15), 2)).setEndValue(endValue);
      return new Promise(function (resolve, reject) {
        spring.addOneTimeListener({
          onSpringAtRest: function onSpringAtRest(spring) {
            // if not 1/0, assume this has been interrupted by subsequent gesture animation
            if (spring.getCurrentValue() !== endValue) {
              return reject();
            }

            resolve(); // otherwise transforms will be removed from cancelled flip causing jitter

            endValue === 1 && onAnimationEnd();
            cleanUpAnimation(inProgressAnimations, flipId);
          }
        });
      });
    });
    Promise.all(springsCompleted).then(onFinished);
  };
};

var finishFlip = makeFinishFunction(1);
var cancelFlip = makeFinishFunction(0);

var updateSprings = function updateSprings(_ref3) {
  var inProgressAnimations = _ref3.inProgressAnimations,
      percentage = _ref3.percentage;
  Object.keys(inProgressAnimations).forEach(function (flipId) {
    var spring = inProgressAnimations[flipId].spring;

    if (!Object.keys(spring._cachedSpringConfig).length) {
      inProgressAnimations[flipId].spring._cachedSpringConfig = inProgressAnimations[flipId].spring._springConfig;
    }

    spring.setSpringConfig({
      tension: 300,
      friction: 26
    });
    inProgressAnimations[flipId].spring.setEndValue(percentage);
  });
};

var getDirection = function getDirection(deltaX, deltaY) {
  if (Math.abs(deltaX) > Math.abs(deltaY)) {
    return deltaX > 0 ? 'right' : 'left';
  }

  return deltaY > 0 ? 'down' : 'up';
};

var Swipe =
/*#__PURE__*/
function () {
  // prevent people from using the same gesture to do multiple things
  // force a mouse up
  function Swipe(props) {
    _classCallCheck(this, Swipe);

    this.props = props;

    _defineProperty(this, "set", void 0);

    _defineProperty(this, "handlers", void 0);

    _defineProperty(this, "flipInitiatorData", undefined);

    _defineProperty(this, "isFinishing", false);

    _defineProperty(this, "prevProps", {});

    var _gestureHandlers = (0, _gestureHandlers2.default)({
      onAction: this.onAction.bind(this),
      config: this.props
    }),
        handlers = _gestureHandlers.handlers,
        set = _gestureHandlers.set;

    this.handlers = handlers;
    this.set = set;
  }

  _createClass(Swipe, [{
    key: "onAction",
    value: function onAction(_ref4) {
      var _this = this;

      var velocity = _ref4.velocity,
          _ref4$delta = _slicedToArray(_ref4.delta, 2),
          deltaX = _ref4$delta[0],
          deltaY = _ref4$delta[1],
          down = _ref4.down,
          first = _ref4.first;

      if (this.isFinishing) return;
      if (first && deltaX + deltaY === 0) return;

      var _this$props = this.props,
          inProgressAnimations = _this$props.inProgressAnimations,
          setIsGestureInitiated = _this$props.setIsGestureInitiated,
          rest = _objectWithoutProperties(_this$props, ["inProgressAnimations", "setIsGestureInitiated"]);

      var flipId = String(this.props.flipId);

      if (!inProgressAnimations || !setIsGestureInitiated) {
        if (false) {
          // eslint-disable-next-line no-console
          console.error('[react-flip-toolkit] No parent Flipper component detected for this Swipe component');
        }

        return;
      }

      var generalFlipInProgress = Boolean(Object.keys(inProgressAnimations).length);

      var onFlipCancelled = function onFlipCancelled() {
        _this.flipInitiatorData && _this.flipInitiatorData.cancelFlip({
          props: _this.props,
          prevProps: _this.prevProps
        });
        delete _this.flipInitiatorData;
        _this.isFinishing = false;
      }; // TODO: figure out why the typings don't work :/


      var config = ['left', 'right', 'up', 'down'].map(function (direction) {
        // @ts-ignore
        if (!rest[direction]) return null;
        return Object.assign({
          direction: direction,
          threshold: _this.props.threshold || defaultCompleteThreshhold
        }, // @ts-ignore
        rest[direction]);
      }).filter(Boolean);

      var initiateGestureControlledFLIP = function initiateGestureControlledFLIP(configMatchingCurrentDirection) {
        // we have to cache all config BEFORE calling initFlip
        // which can dramatically change the UI and/or the FLIP config
        // that the component has
        // copying it for safety but maybe that isnt necessary
        _this.flipInitiatorData = Object.assign({}, configMatchingCurrentDirection); // reset the initial arg so that we are calculating gesture delta from the current mouse position
        // this will make interrupted/edge case scenarios smoother

        _this.set(function (state) {
          return Object.assign({}, state, {
            initial: state.xy
          });
        });

        setIsGestureInitiated(true);
        configMatchingCurrentDirection.initFlip({
          props: _this.props,
          prevProps: _this.prevProps
        }); // must call this func this after getSnapshotBeforeUpdate has run!

        var afterFLIPHasBeenInitiated = function afterFLIPHasBeenInitiated() {
          // maybe gesture occurred but component didn't change posiition
          if (!inProgressAnimations[flipId]) {
            delete _this.flipInitiatorData;
            return;
          }

          Object.keys(inProgressAnimations).forEach(function (inProgressAnimationFlipId) {
            if (flipId) {
              // @ts-ignore
              inProgressAnimations[inProgressAnimationFlipId].flipInitiator = flipId;
            }
          });
        };

        setTimeout(afterFLIPHasBeenInitiated, 0);
      };

      var currentDirection = getDirection(deltaX, deltaY);
      var configMatchingCurrentDirection = config.filter(function (configEntry) {
        return configEntry.direction === currentDirection;
      })[0];

      if (!generalFlipInProgress) {
        if (!configMatchingCurrentDirection) {
          return;
        }

        return initiateGestureControlledFLIP(configMatchingCurrentDirection);
      }

      if (!this.flipInitiatorData) {
        return;
      }

      var returnToUnFlippedState = function returnToUnFlippedState() {
        _this.isFinishing = true;
        cancelFlip({
          velocity: velocity,
          inProgressAnimations: inProgressAnimations,
          onFinished: onFlipCancelled
        });
      };

      if (this.flipInitiatorData.direction !== currentDirection) {
        // user might have done a mouseup while moving in another direction
        if (generalFlipInProgress && !down) {
          return returnToUnFlippedState();
        }
      }

      var absoluteMovement = getMovementScalar({
        deltaX: deltaX,
        deltaY: deltaY,
        direction: this.flipInitiatorData.direction
      }); // must make sure not to call this when the user is starting a new gesture,
      // only when finger has not been lifted

      if (absoluteMovement <= 0 && !first) {
        return onFlipCancelled();
      }

      var _inProgressAnimations2 = inProgressAnimations[flipId].difference,
          translateXDifference = _inProgressAnimations2.translateXDifference,
          translateYDifference = _inProgressAnimations2.translateYDifference,
          prevRect = _inProgressAnimations2.prevRect,
          currentRect = _inProgressAnimations2.currentRect;
      var difference = Math.abs(['up', 'down'].indexOf(this.flipInitiatorData.direction) > -1 ? currentRect.height - prevRect.height + translateYDifference : currentRect.width - currentRect.width + translateXDifference);
      var percentage = absoluteMovement / difference; // cancel flip -- this is interruptible if user
      // tries to drag before animation is completed

      if (!down && percentage < this.flipInitiatorData.threshold) {
        return returnToUnFlippedState();
      } // gesture has gone far enough, animation can complete


      if (percentage > this.flipInitiatorData.threshold) {
        this.isFinishing = true;
        return finishFlip({
          velocity: velocity,
          inProgressAnimations: inProgressAnimations,
          onFinished: function onFinished() {
            delete _this.flipInitiatorData;
            delete _this.isFinishing;
          }
        });
      }

      updateSprings({
        percentage: percentage,
        inProgressAnimations: inProgressAnimations
      });
    }
  }]);

  return Swipe;
}();

var _default = Swipe;
exports.default = _default;