"use strict";

var _sinon = _interopRequireDefault(require("sinon"));

var Rematrix = _interopRequireWildcard(require("rematrix"));

var _index = _interopRequireDefault(require("../index"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var testEl = document.querySelector('#test');

var getElement = function getElement(id) {
  return testEl.querySelector("[data-flip-id=".concat(id, "]"));
};

var scopedSelector = function scopedSelector(selector) {
  return _toConsumableArray(testEl.querySelectorAll(selector));
};

describe('animateFlippedElements', function () {
  it('should return a function that calls onStart with reference to the element on the first tick of the animation', function (done) {
    testEl.innerHTML = " <div>\n    <div data-flip-id=\"id-1\" data-flip-config='{}'></div>\n    </div>\n  ";
    var inProgressAnimations = {};

    var onStart = _sinon.default.fake();

    var flip = (0, _index.default)({
      applyTransformOrigin: true,
      flippedIds: ['id-1'],
      flipCallbacks: {
        'id-1': {
          onStart: onStart
        }
      },
      inProgressAnimations: inProgressAnimations,
      flippedElementPositionsBeforeUpdate: {
        'id-1': {
          rect: {
            top: 10,
            left: 10,
            width: 100,
            height: 100
          }
        }
      },
      flippedElementPositionsAfterUpdate: {
        'id-1': {
          element: getElement('id-1'),
          rect: {
            top: 100,
            left: 100,
            width: 100,
            height: 100,
            bottom: 200,
            right: 200
          }
        }
      },
      decisionData: {},
      getElement: getElement,
      scopedSelector: scopedSelector
    });
    flip();
    setTimeout(function () {
      expect(onStart.callCount).to.equal(1);
      expect(onStart.args[0][0]).to.equal(getElement('id-1'));
      done();
    }, 10);
  });
  it('should preserve transforms in the final state of the element', function (done) {
    testEl.innerHTML = " <div>\n    <div data-flip-id=\"id-1\" class=\"visible-block has-transform\" data-flip-config='{\"translate\":true,\"scale\":true,\"opacity\":true}'></div>\n    <div data-flip-id=\"id-2\" class=\"visible-block\" data-flip-config='{\"translate\":true,\"scale\":true,\"opacity\":true}'></div>\n    </div>\n  ";
    var firstElementTransform = getComputedStyle(getElement('id-1')).transform;
    var flip = (0, _index.default)({
      flippedIds: ['id-1', 'id-2'],
      flipCallbacks: {},
      inProgressAnimations: {},
      flippedElementPositionsBeforeUpdate: {
        'id-1': {
          rect: {
            top: 100,
            left: 100,
            width: 100,
            height: 100,
            bottom: 200,
            right: 200
          }
        },
        'id-2': {
          rect: {
            top: 100,
            left: 100,
            width: 100,
            height: 100,
            bottom: 200,
            right: 200
          }
        }
      },
      flippedElementPositionsAfterUpdate: {
        'id-1': {
          element: getElement('id-1'),
          rect: getElement('id-1').getBoundingClientRect(),
          transform: firstElementTransform
        },
        'id-2': {
          element: getElement('id-2'),
          rect: getElement('id-2').getBoundingClientRect()
        }
      },
      getElement: getElement,
      scopedSelector: scopedSelector
    });
    flip();
    setTimeout(function () {
      var newTransform = Rematrix.parse(getComputedStyle(getElement('id-1')).transform).map(function (n) {
        return Math.floor(n);
      });
      expect(newTransform).to.deep.equal([2, 0, 0, 0, 0, 2, 0, 0, 0, 0, 1, 0, 50, 25, 0, 1]);
      done();
    }, 1000);
  });
  it('should apply a custom transform origin if one is provided in the flip config', function () {
    testEl.innerHTML = " <div>\n    <div data-flip-id=\"id-1\" data-flip-config='{\"transformOrigin\": \"75% 75%\", \"translate\":true}'></div>\n    </div>\n  ";
    (0, _index.default)({
      flippedIds: ['id-1'],
      flipCallbacks: {},
      inProgressAnimations: {},
      duration: 1,
      flippedElementPositionsBeforeUpdate: {
        'id-1': {
          rect: {
            top: 10,
            left: 10,
            width: 100,
            height: 100
          }
        }
      },
      flippedElementPositionsAfterUpdate: {
        'id-1': {
          element: getElement('id-1'),
          rect: {
            top: 100,
            left: 100,
            width: 100,
            height: 100,
            bottom: 200,
            right: 200
          }
        }
      },
      getElement: getElement,
      scopedSelector: scopedSelector
    });
    expect(getElement('id-1').style.transformOrigin).to.equal('75% 75%');
  });
  it('should otherwise apply a 0 0 transform origin', function () {
    testEl.innerHTML = " <div>\n    <div data-flip-id=\"id-1\" data-flip-config='{}'></div>\n    </div>\n  ";
    (0, _index.default)({
      applyTransformOrigin: true,
      flippedIds: ['id-1'],
      flipCallbacks: {},
      inProgressAnimations: {},
      duration: 1,
      flippedElementPositionsBeforeUpdate: {
        'id-1': {
          rect: {
            top: 10,
            left: 10,
            width: 100,
            height: 100
          }
        }
      },
      flippedElementPositionsAfterUpdate: {
        element: getElement('id-1'),
        'id-1': {
          element: getElement('id-1'),
          rect: {
            top: 100,
            left: 100,
            width: 100,
            height: 100,
            bottom: 200,
            right: 200
          }
        }
      },
      getElement: getElement,
      scopedSelector: scopedSelector
    });
    expect(getElement('id-1').style.transformOrigin).to.equal('0px 0px');
  });
  it('should cache stop and onComplete functions for the element in inProgressAnimations ', function () {
    testEl.innerHTML = " <div>\n    <div data-flip-id=\"id-1\" data-flip-config='{}'></div>\n    </div>\n  ";
    var inProgressAnimations = {};
    var flip = (0, _index.default)({
      applyTransformOrigin: true,
      flippedIds: ['id-1'],
      flipCallbacks: {
        'id-1': {
          onComplete: function onComplete() {}
        }
      },
      inProgressAnimations: inProgressAnimations,
      flippedElementPositionsBeforeUpdate: {
        'id-1': {
          rect: {
            top: 10,
            left: 10,
            width: 100,
            height: 100
          }
        }
      },
      flippedElementPositionsAfterUpdate: {
        'id-1': {
          element: getElement('id-1'),
          rect: {
            top: 100,
            left: 100,
            width: 100,
            height: 100,
            bottom: 200,
            right: 200
          }
        }
      },
      getElement: getElement,
      scopedSelector: scopedSelector
    });
    flip();
    expect(_typeof(inProgressAnimations['id-1'].spring.destroy)).to.equal('function');
    expect(_typeof(inProgressAnimations['id-1'].onAnimationEnd)).to.equal('function');
  });
  it('should return a function that, when called, returns a promise resolved with all flip ids', function (done) {
    testEl.innerHTML = " <div>\n    <div data-flip-id=\"id-1\" data-flip-config='{}'></div>\n    </div>\n  ";
    var inProgressAnimations = {};
    var flip = (0, _index.default)({
      applyTransformOrigin: true,
      flippedIds: ['id-1'],
      flipCallbacks: {
        'id-1': {
          onComplete: function onComplete() {}
        }
      },
      inProgressAnimations: inProgressAnimations,
      flippedElementPositionsBeforeUpdate: {
        'id-1': {
          rect: {
            top: 10,
            left: 10,
            width: 100,
            height: 100
          }
        }
      },
      flippedElementPositionsAfterUpdate: {
        'id-1': {
          element: getElement('id-1'),
          rect: {
            top: 100,
            left: 100,
            width: 100,
            height: 100,
            bottom: 200,
            right: 200
          }
        }
      },
      getElement: getElement,
      scopedSelector: scopedSelector
    });
    flip().then(function (flipIds) {
      expect(flipIds).to.deep.equal(['id-1']);
      done();
    });
  });
  it('should handle the case when there are no flipIds ', function (done) {
    testEl.innerHTML = " <div>\n    </div>\n  ";
    var flip = (0, _index.default)({
      applyTransformOrigin: true,
      flippedIds: [],
      flipCallbacks: {},
      inProgressAnimations: {},
      duration: 1,
      flippedElementPositionsBeforeUpdate: {
        'id-1': {
          rect: {
            top: 10,
            left: 10,
            width: 100,
            height: 100
          }
        }
      },
      flippedElementPositionsAfterUpdate: {},
      getElement: getElement,
      scopedSelector: scopedSelector
    });
    flip().then(function (flipIds) {
      expect(flipIds).to.deep.equal([]);
      done();
    });
  });
});