"use strict";

var _index = require("../index");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var testEl = document.querySelector('#test');
describe('invertTransformsForChildren', function () {
  it('should ignore children that do not exist in the DOM', function () {
    testEl.innerHTML = "\n    <div data-flip-id=\"id-1\" >\n    <div data-inverse-flip-id=\"id-1\" style=\"height:100px; width: 100px; opacity: .5\"></div>\n    </div>\n  ";
    var orphanedEl = document.createElement('div');
    var invertedChildren = [document.querySelector('[data-inverse-flip-id="id-1"]'), orphanedEl].map(function (el) {
      return [el, {
        translate: true
      }];
    });
    var matrix = [1.5, null, null, 2, 25, 35];
    var body = document.querySelector('body');
    (0, _index.invertTransformsForChildren)({
      invertedChildren: invertedChildren,
      matrix: matrix,
      body: body
    });
    expect(invertedChildren[0][0].style.transform).to.equal('translate(-16.6667px, -17.5px)');
    expect(invertedChildren[1][0].style.transform).to.equal('');
  });
  it('should apply a counter translation to children that request it', function () {
    testEl.innerHTML = "\n    <div data-flip-id=\"id-1\" >\n    <div data-inverse-flip-id=\"id-1\" style=\"height:100px; width: 100px; opacity: .5\"></div>\n    <div data-inverse-flip-id=\"id-1\" style=\"height:200px; width: 100px;\"></div>\n    </div>\n  ";

    var invertedChildren = _toConsumableArray(document.querySelectorAll('[data-inverse-flip-id="id-1"]')).map(function (el, i) {
      return [el, {
        translate: i === 0 ? true : false
      }];
    });

    var matrix = [1, null, null, 2, 25, 20];
    var body = document.querySelector('body');
    (0, _index.invertTransformsForChildren)({
      invertedChildren: invertedChildren,
      matrix: matrix,
      body: body
    });
    expect(invertedChildren[0][0].style.transform).to.equal('translate(-25px, -10px)');
    expect(invertedChildren[1][0].style.transform).to.equal('');
  });
  it('should apply a counter scale to children that request it', function () {
    testEl.innerHTML = "\n    <div data-flip-id=\"id-1\" >\n    <div data-inverse-flip-id=\"id-1\" style=\"height:100px; width: 100px; opacity: .5\"></div>\n    <div data-inverse-flip-id=\"id-1\" style=\"height:200px; width: 100px;\"></div>\n    </div>\n  ";

    var invertedChildren = _toConsumableArray(document.querySelectorAll('[data-inverse-flip-id="id-1"]')).map(function (el, i) {
      return [el, {
        scale: i === 0 ? true : false
      }];
    });

    var matrix = [1.5, null, null, 2, 25, 35];
    var body = document.querySelector('body');
    (0, _index.invertTransformsForChildren)({
      invertedChildren: invertedChildren,
      matrix: matrix,
      body: body
    });
    expect(invertedChildren[0][0].style.transform).to.equal('scale(0.666667, 0.5)');
    expect(invertedChildren[1][0].style.transform).to.equal('');
  });
  it('should apply both a counter scale and a counter transform to children that request it', function () {
    testEl.innerHTML = "\n    <div data-flip-id=\"id-1\" >\n    <div data-inverse-flip-id=\"id-1\" style=\"height:100px; width: 100px; opacity: .5\"></div>\n    <div data-inverse-flip-id=\"id-1\" style=\"height:200px; width: 100px;\"></div>\n    </div>\n  ";

    var invertedChildren = _toConsumableArray(document.querySelectorAll('[data-inverse-flip-id="id-1"]')).map(function (el, i) {
      return [el, {
        scale: i === 0 ? true : false,
        translate: i === 0 ? true : false
      }];
    });

    var matrix = [1.5, null, null, 2, 25, 35];
    var body = document.querySelector('body');
    (0, _index.invertTransformsForChildren)({
      invertedChildren: invertedChildren,
      matrix: matrix,
      body: body
    });
    expect(invertedChildren[0][0].style.transform).to.equal('translate(-16.6667px, -17.5px) scale(0.666667, 0.5)');
    expect(invertedChildren[1][0].style.transform).to.equal('');
  });
});
describe('createApplyStylesFunc', function () {
  it('should create a function to apply the matrix transform to the element', function () {
    testEl.innerHTML = "\n    <div data-flip-id=\"id-1\" >\n    </div>\n  ";
    var body = document.querySelector('body');
    var element = document.querySelector('[data-flip-id="id-1"]');
    var invertedChildren = [];
    var applyStyles = (0, _index.createApplyStylesFunc)({
      element: element,
      invertedChildren: invertedChildren,
      body: body
    });
    applyStyles({
      matrix: [0.2, 0, 0, 0.5, -300, 250]
    });
    expect(element.style.transform).to.equal('matrix(0.2, 0, 0, 0.5, -300, 250)');
  });
  it('that function should apply the opacity style to the element if opacity is a number', function () {
    testEl.innerHTML = "\n    <div data-flip-id=\"id-1\" >\n    </div>\n  ";
    var body = document.querySelector('body');
    var element = document.querySelector('[data-flip-id="id-1"]');
    var invertedChildren = [];
    var applyStyles = (0, _index.createApplyStylesFunc)({
      element: element,
      invertedChildren: invertedChildren,
      body: body
    });
    applyStyles({
      matrix: [0.2, 0, 0, 0.5, -300, 250],
      opacity: 'invalid opacity value here'
    });
    expect(element.style.opacity).to.equal('');
    applyStyles({
      matrix: [0.2, 0, 0, 0.5, -300, 250],
      opacity: 0.5
    });
    expect(element.style.opacity).to.equal('0.5');
  });
  it('that function should apply a min-width of 1px and/or min-height of 1px if the forceMinWidth or forceMinHeight args are provided ', function () {
    testEl.innerHTML = "\n    <div data-flip-id=\"id-1\" >\n    </div>\n  ";
    var body = document.querySelector('body');
    var element = document.querySelector('[data-flip-id="id-1"]');
    var invertedChildren = [];
    var applyStyles = (0, _index.createApplyStylesFunc)({
      element: element,
      invertedChildren: invertedChildren,
      body: body
    });
    expect(element.style.minWidth).to.equal('');
    expect(element.style.minHeight).to.equal('');
    applyStyles({
      matrix: [0.2, 0, 0, 0.5, -300, 250],
      forceMinVals: true
    });
    expect(element.style.minWidth).to.equal('1px');
    expect(element.style.minHeight).to.equal('1px');
  });
  it('should apply the identity transform if requested to do so', function () {
    var body = document.querySelector('body');
    var element = document.querySelector('[data-flip-id="id-1"]');
    var invertedChildren = [];
    var applyStyles = (0, _index.createApplyStylesFunc)({
      element: element,
      invertedChildren: invertedChildren,
      body: body
    });
    applyStyles({
      matrix: [1, 0, 0, 1, 0, 0]
    });
    expect(element.style.transform).to.equal('matrix(1, 0, 0, 1, 0, 0)');
  });
});