"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.rectInViewport = exports.createApplyStylesFunc = exports.invertTransformsForChildren = exports.convertMatrix2dArrayToString = exports.convertMatrix3dArrayTo2dArray = void 0;

var Rematrix = _interopRequireWildcard(require("rematrix"));

var _springSettings = require("../../springSettings");

var _utilities = require("../../utilities");

var constants = _interopRequireWildcard(require("../../constants"));

var _spring = require("./spring");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

// 3d transforms were causing weird issues in chrome,
// especially when opacity was also being tweened,
// so convert to a 2d matrix
var convertMatrix3dArrayTo2dArray = function convertMatrix3dArrayTo2dArray(matrix) {
  return [0, 1, 4, 5, 12, 13].map(function (index) {
    return matrix[index];
  });
};

exports.convertMatrix3dArrayTo2dArray = convertMatrix3dArrayTo2dArray;

var convertMatrix2dArrayToString = function convertMatrix2dArrayToString(matrix) {
  return "matrix(".concat(matrix.join(', '), ")");
};

exports.convertMatrix2dArrayToString = convertMatrix2dArrayToString;

var invertTransformsForChildren = function invertTransformsForChildren(_ref) {
  var invertedChildren = _ref.invertedChildren,
      matrix = _ref.matrix,
      body = _ref.body;
  invertedChildren.forEach(function (_ref2) {
    var _ref3 = _slicedToArray(_ref2, 2),
        child = _ref3[0],
        childFlipConfig = _ref3[1];

    if (!body.contains(child)) {
      return;
    }

    var scaleX = matrix[0];
    var scaleY = matrix[3];
    var translateX = matrix[4];
    var translateY = matrix[5];
    var inverseVals = {
      translateX: 0,
      translateY: 0,
      scaleX: 1,
      scaleY: 1
    };
    var transformString = '';

    if (childFlipConfig.translate) {
      inverseVals.translateX = -translateX / scaleX;
      inverseVals.translateY = -translateY / scaleY;
      transformString += "translate(".concat(inverseVals.translateX, "px, ").concat(inverseVals.translateY, "px)");
    }

    if (childFlipConfig.scale) {
      inverseVals.scaleX = 1 / scaleX;
      inverseVals.scaleY = 1 / scaleY;
      transformString += " scale(".concat(inverseVals.scaleX, ", ").concat(inverseVals.scaleY, ")");
    }

    child.style.transform = transformString;
  });
};

exports.invertTransformsForChildren = invertTransformsForChildren;

var createApplyStylesFunc = function createApplyStylesFunc(_ref4) {
  var element = _ref4.element,
      invertedChildren = _ref4.invertedChildren,
      body = _ref4.body;
  return function (_ref5) {
    var matrix = _ref5.matrix,
        opacity = _ref5.opacity,
        forceMinVals = _ref5.forceMinVals;

    if ((0, _utilities.isNumber)(opacity)) {
      element.style.opacity = opacity + '';
    }

    if (forceMinVals) {
      element.style.minHeight = '1px';
      element.style.minWidth = '1px';
    }

    if (!matrix) {
      return;
    }

    var stringTransform = convertMatrix2dArrayToString(matrix); // always apply transform, even if identity,
    // because identity might be the starting state in a FLIP
    // transition, if the element's position is controlled by transforms

    element.style.transform = stringTransform;

    if (invertedChildren) {
      invertTransformsForChildren({
        invertedChildren: invertedChildren,
        matrix: matrix,
        body: body
      });
    }
  };
};

exports.createApplyStylesFunc = createApplyStylesFunc;

var rectInViewport = function rectInViewport(_ref6) {
  var top = _ref6.top,
      bottom = _ref6.bottom,
      left = _ref6.left,
      right = _ref6.right;
  return top < window.innerHeight && bottom > 0 && left < window.innerWidth && right > 0;
};

exports.rectInViewport = rectInViewport;

var getInvertedChildren = function getInvertedChildren(element, id) {
  return (0, _utilities.toArray)(element.querySelectorAll("[".concat(constants.DATA_INVERSE_FLIP_ID, "=\"").concat(id, "\"]")));
};

var _default = function _default(_ref7) {
  var flippedIds = _ref7.flippedIds,
      flipCallbacks = _ref7.flipCallbacks,
      inProgressAnimations = _ref7.inProgressAnimations,
      flippedElementPositionsBeforeUpdate = _ref7.flippedElementPositionsBeforeUpdate,
      flippedElementPositionsAfterUpdate = _ref7.flippedElementPositionsAfterUpdate,
      applyTransformOrigin = _ref7.applyTransformOrigin,
      spring = _ref7.spring,
      getElement = _ref7.getElement,
      debug = _ref7.debug,
      _ref7$staggerConfig = _ref7.staggerConfig,
      staggerConfig = _ref7$staggerConfig === void 0 ? {} : _ref7$staggerConfig,
      _ref7$decisionData = _ref7.decisionData,
      decisionData = _ref7$decisionData === void 0 ? {} : _ref7$decisionData,
      onComplete = _ref7.onComplete,
      containerEl = _ref7.containerEl,
      isGestureControlled = _ref7.isGestureControlled;
  // the stuff below is used so we can return a promise that resolves when all FLIP animations have
  // completed
  var closureResolve;
  var flipCompletedPromise = new Promise(function (resolve) {
    closureResolve = resolve;
  }); // hook for users of lib to attach logic when all flip animations have completed

  if (onComplete) {
    flipCompletedPromise.then(function () {
      return onComplete(containerEl, decisionData);
    });
  }

  if (!flippedIds.length) {
    return function () {
      closureResolve([]);
      return flipCompletedPromise;
    };
  }

  var completedAnimationIds = [];
  var firstElement = getElement(flippedIds[0]); // special handling for iframes

  var body = firstElement ? firstElement.ownerDocument.querySelector('body') : document.querySelector('body');

  if (false) {
    if (debug) {
      // eslint-disable-next-line no-console
      console.error('[react-flip-toolkit]\nThe "debug" prop is set to true. All FLIP animations will return at the beginning of the transition.');
    }
  }

  var duplicateFlipIds = (0, _utilities.getDuplicateValsAsStrings)(flippedIds);

  if (false) {
    if (duplicateFlipIds.length) {
      // eslint-disable-next-line no-console
      console.error("[react-flip-toolkit]\nThere are currently multiple elements with the same flipId on the page.\nThe animation will only work if each Flipped component has a unique flipId.\nDuplicate flipId".concat(duplicateFlipIds.length > 1 ? 's' : '', ": ").concat(duplicateFlipIds.join('\n')));
    }
  }

  var flipDataArray = flippedIds // take all the measurements we need
  // and return an object with animation functions + necessary data
  .map(function (id) {
    var prevRect = flippedElementPositionsBeforeUpdate[id].rect;
    var currentRect = flippedElementPositionsAfterUpdate[id].rect;
    var prevOpacity = flippedElementPositionsBeforeUpdate[id].opacity;
    var currentOpacity = flippedElementPositionsAfterUpdate[id].opacity;
    var needsForcedMinVals = currentRect.width < 1 || currentRect.height < 1;
    var element = flippedElementPositionsAfterUpdate[id].element; // don't animate elements outside of the user's viewport

    if (!rectInViewport(prevRect) && !rectInViewport(currentRect)) {
      return false;
    } // this might happen if we are rapidly adding & removing elements(?)


    if (!element) {
      return false;
    }

    var flipConfig = JSON.parse(element.dataset.flipConfig);
    var springConfig = (0, _springSettings.getSpringConfig)({
      flipperSpring: spring,
      flippedSpring: flipConfig.spring
    });
    var stagger = flipConfig.stagger === true ? 'default' : flipConfig.stagger;
    var toReturn = {
      element: element,
      id: id,
      stagger: stagger,
      springConfig: springConfig
    };

    if (flipCallbacks[id] && flipCallbacks[id].shouldFlip) {
      var elementShouldFlip = flipCallbacks[id].shouldFlip(decisionData.previous, decisionData.current);

      if (!elementShouldFlip) {
        return false;
      }
    } // don't animate elements that didn't visibly change
    // but possibly animate their children


    var translateXDifference = Math.abs(prevRect.left - currentRect.left);
    var translateYDifference = Math.abs(prevRect.top - currentRect.top);
    var translateDifference = translateXDifference + translateYDifference;
    var scaleXDifference = Math.abs(prevRect.width - currentRect.width);
    var scaleYDifference = Math.abs(prevRect.height - currentRect.height);
    var scaleDifference = scaleXDifference + scaleYDifference;
    var opacityDifference = Math.abs(currentOpacity - prevOpacity);

    if (translateDifference < 0.5 && scaleDifference < 0.5 && opacityDifference < 0.01) {
      return false;
    }

    var currentTransform = Rematrix.parse(flippedElementPositionsAfterUpdate[id].transform);
    var toVals = {
      matrix: currentTransform
    };
    var fromVals = {
      matrix: []
    };
    var transformsArray = [currentTransform]; // we're only going to animate the values that the child wants animated

    if (flipConfig.translate) {
      transformsArray.push(Rematrix.translateX(prevRect.left - currentRect.left));
      transformsArray.push(Rematrix.translateY(prevRect.top - currentRect.top));
    } // going any smaller than 1px breaks transitions in Chrome


    if (flipConfig.scale) {
      transformsArray.push(Rematrix.scaleX(Math.max(prevRect.width, 1) / Math.max(currentRect.width, 1)));
      transformsArray.push(Rematrix.scaleY(Math.max(prevRect.height, 1) / Math.max(currentRect.height, 1)));
    }

    if (flipConfig.opacity) {
      fromVals.opacity = prevOpacity;
      toVals.opacity = currentOpacity;
    }

    var invertedChildren = [];

    if (!flipCallbacks[id] || !flipCallbacks[id].shouldInvert || flipCallbacks[id].shouldInvert(decisionData.previous, decisionData.current)) {
      var invertedChildElements = getInvertedChildren(element, id);
      invertedChildren = invertedChildElements.map(function (c) {
        return [c, JSON.parse(c.dataset.flipConfig)];
      });
    }

    fromVals.matrix = convertMatrix3dArrayTo2dArray(transformsArray.reduce(Rematrix.multiply));
    toVals.matrix = convertMatrix3dArrayTo2dArray(toVals.matrix);
    var applyStyles = createApplyStylesFunc({
      element: element,
      invertedChildren: invertedChildren,
      body: body
    });
    var onComplete;

    if (flipCallbacks[id] && flipCallbacks[id].onComplete) {
      // must cache or else this could cause an error
      var cachedOnComplete = flipCallbacks[id].onComplete;

      onComplete = function onComplete() {
        return cachedOnComplete(element, decisionData);
      };
    } // this should be called when animation ends naturally
    // but also when it is interrupted
    // when it is called, the animation has already been cancelled


    var onAnimationEnd = function onAnimationEnd() {
      delete inProgressAnimations[id];

      if ((0, _utilities.isFunction)(onComplete)) {
        onComplete();
      } // remove identity transform -- this should have no effect on layout


      element.style.transform = '';
      invertedChildren.forEach(function (_ref8) {
        var _ref9 = _slicedToArray(_ref8, 1),
            element = _ref9[0];

        element.style.transform = '';
      });

      if (needsForcedMinVals && element) {
        element.style.minHeight = '';
        element.style.minWidth = '';
      }

      completedAnimationIds.push(id);

      if (completedAnimationIds.length >= flipDataArray.length) {
        // we can theoretically call multiple times since a promise only resolves 1x
        // but that shouldnt happen
        closureResolve(completedAnimationIds);
      }
    };

    var animateOpacity = (0, _utilities.isNumber)(fromVals.opacity) && (0, _utilities.isNumber)(toVals.opacity) && fromVals.opacity !== toVals.opacity;
    var onStartCalled = false;

    var getOnUpdateFunc = function getOnUpdateFunc(_ref10) {
      var spring = _ref10.spring,
          onAnimationEnd = _ref10.onAnimationEnd;
      inProgressAnimations[id] = {
        // only for gesture control
        // @ts-ignore
        spring: spring,
        onAnimationEnd: onAnimationEnd,
        difference: {
          translateXDifference: translateXDifference,
          translateYDifference: translateYDifference,
          prevRect: prevRect,
          currentRect: currentRect
        }
      };

      var onUpdate = function onUpdate(spring) {
        if (flipCallbacks[id] && flipCallbacks[id].onSpringUpdate) {
          flipCallbacks[id].onSpringUpdate(spring.getCurrentValue());
        } // trigger the user provided onStart function


        if (!onStartCalled) {
          onStartCalled = true;

          if (flipCallbacks[id] && flipCallbacks[id].onStart) {
            flipCallbacks[id].onStart(element, decisionData);
          }
        }

        var currentValue = spring.getCurrentValue();

        if (!body.contains(element)) {
          spring.destroy();
          return;
        }

        var vals = {
          matrix: []
        };
        vals.matrix = fromVals.matrix.map(function (fromVal, index) {
          return (0, _utilities.tweenProp)(fromVal, toVals.matrix[index], currentValue);
        });

        if (animateOpacity) {
          vals.opacity = (0, _utilities.tweenProp)(fromVals.opacity, toVals.opacity, currentValue);
        }

        applyStyles(vals);
      };

      return onUpdate;
    };

    var initializeFlip = function initializeFlip() {
      // before animating, immediately apply FLIP styles to prevent flicker
      applyStyles({
        matrix: fromVals.matrix,
        opacity: animateOpacity ? fromVals.opacity : undefined,
        forceMinVals: needsForcedMinVals
      });

      if (flipCallbacks[id] && flipCallbacks[id].onStartImmediate) {
        flipCallbacks[id].onStartImmediate(element, decisionData);
      } // and batch any other style updates if necessary


      if (flipConfig.transformOrigin) {
        element.style.transformOrigin = flipConfig.transformOrigin;
      } else if (applyTransformOrigin) {
        element.style.transformOrigin = '0 0';
      }

      invertedChildren.forEach(function (_ref11) {
        var _ref12 = _slicedToArray(_ref11, 2),
            child = _ref12[0],
            childFlipConfig = _ref12[1];

        if (childFlipConfig.transformOrigin) {
          child.style.transformOrigin = childFlipConfig.transformOrigin;
        } else if (applyTransformOrigin) {
          child.style.transformOrigin = '0 0';
        }
      });
    };

    return (0, _utilities.assign)({}, toReturn, {
      stagger: stagger,
      springConfig: springConfig,
      getOnUpdateFunc: getOnUpdateFunc,
      initializeFlip: initializeFlip,
      onAnimationEnd: onAnimationEnd,
      isGestureControlled: isGestureControlled,
      delayUntil: flipConfig.delayUntil
    });
  }) // filter out data for all non-animated elements first
  .filter(Boolean);
  flipDataArray.forEach(function (_ref13) {
    var initializeFlip = _ref13.initializeFlip;
    return initializeFlip();
  });

  if (debug) {
    return function () {};
  }

  var elementIsFlipped = function elementIsFlipped(flipId) {
    return flipDataArray.filter(function (f) {
      return f.id === flipId;
    }).length;
  };

  var delayedFlip = flipDataArray.filter(function (f) {
    return f.delayUntil && elementIsFlipped(f.delayUntil);
  }); // key: flipId value: flip to delay until key is called

  var delayUntilSprings = {}; // key: flipId value: stagger to delay until key is called

  var delayUntilStaggers = {}; // key: stagger value: true

  var delayedStaggerKeys = {};
  delayedFlip.forEach(function (flip) {
    if (flip.stagger) {
      delayedStaggerKeys[flip.stagger] = true;
      if (delayUntilStaggers[flip.delayUntil]) delayUntilStaggers[flip.delayUntil].push(flip.stagger);else delayUntilStaggers[flip.delayUntil] = [flip.stagger];
    } else {
      if (delayUntilSprings[flip.delayUntil]) delayUntilSprings[flip.delayUntil].push(flip);else delayUntilSprings[flip.delayUntil] = [flip];
    }
  });
  var staggerDict = flipDataArray.filter(function (flipData) {
    return flipData.stagger;
  }).reduce(function (acc, curr) {
    if (acc[curr.stagger]) {
      acc[curr.stagger].push(curr);
    } else {
      acc[curr.stagger] = [curr];
    }

    return acc;
  }, {});
  var immediateFlip = flipDataArray.filter(function (f) {
    return delayedFlip.indexOf(f) === -1;
  });
  immediateFlip.forEach(function (flipData) {
    flipData.onSpringActivate = function () {
      if (delayUntilSprings[flipData.id]) {
        delayUntilSprings[flipData.id].forEach(_spring.createSpring);
      }

      if (delayUntilStaggers[flipData.id]) {
        var uniqueStaggerKeys = Object.keys(delayUntilStaggers[flipData.id].reduce(function (acc, curr) {
          return (0, _utilities.assign)(acc, _defineProperty({}, curr, true));
        }, {}));
        uniqueStaggerKeys.forEach(function (staggerKey) {
          (0, _spring.createStaggeredSprings)(staggerDict[staggerKey], staggerConfig[staggerKey], isGestureControlled);
        });
      }
    };
  });
  return function () {
    // if there are no active FLIP animations, immediately resolve the
    // returned promise
    if (!flipDataArray.length) {
      closureResolve([]);
    } // animate non-staggered elements


    immediateFlip.filter(function (flipData) {
      return !flipData.stagger;
    }).forEach(_spring.createSpring); // animate staggered elements

    Object.keys(staggerDict).forEach(function (staggerKey) {
      if (delayedStaggerKeys[staggerKey]) return;
      (0, _spring.createStaggeredSprings)(staggerDict[staggerKey], staggerConfig[staggerKey], isGestureControlled);
    });
    return flipCompletedPromise;
  };
};

exports.default = _default;