"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var animateUnflippedElements = function animateUnflippedElements(_ref) {
  var unflippedIds = _ref.unflippedIds,
      flipCallbacks = _ref.flipCallbacks,
      getElement = _ref.getElement,
      flippedElementPositionsBeforeUpdate = _ref.flippedElementPositionsBeforeUpdate,
      flippedElementPositionsAfterUpdate = _ref.flippedElementPositionsAfterUpdate,
      inProgressAnimations = _ref.inProgressAnimations,
      decisionData = _ref.decisionData;
  var enteringElementIds = unflippedIds.filter(function (id) {
    return flippedElementPositionsAfterUpdate[id];
  });
  var animatedEnteringElementIds = enteringElementIds.filter(function (id) {
    return flipCallbacks[id] && flipCallbacks[id].onAppear;
  });
  var animatedExitingElementIds = unflippedIds.filter(function (id) {
    return flippedElementPositionsBeforeUpdate[id] && flipCallbacks[id] && flipCallbacks[id].onExit;
  });

  var hideEnteringElements = function hideEnteringElements() {
    animatedEnteringElementIds.forEach(function (id) {
      var element = getElement(id);

      if (element) {
        element.style.opacity = '0';
      }
    });
  };

  var animateEnteringElements = function animateEnteringElements() {
    animatedEnteringElementIds.forEach(function (id, i) {
      var element = getElement(id);

      if (element) {
        flipCallbacks[id].onAppear(element, i, decisionData);
      }
    });
  };

  var closureResolve;
  var promiseToReturn = new Promise(function (resolve) {
    closureResolve = resolve;
  });
  var fragmentTuples = [];
  var exitingElementCount = 0;
  var onExitCallbacks = animatedExitingElementIds.map(function (id, i) {
    var _flippedElementPositi = flippedElementPositionsBeforeUpdate[id].domDataForExitAnimations,
        element = _flippedElementPositi.element,
        parent = _flippedElementPositi.parent,
        _flippedElementPositi2 = _flippedElementPositi.childPosition,
        top = _flippedElementPositi2.top,
        left = _flippedElementPositi2.left,
        width = _flippedElementPositi2.width,
        height = _flippedElementPositi2.height; // insert back into dom

    if (getComputedStyle(parent).position === 'static') {
      parent.style.position = 'relative';
    }

    element.style.transform = 'matrix(1, 0, 0, 1, 0, 0)';
    element.style.position = 'absolute';
    element.style.top = top + 'px';
    element.style.left = left + 'px'; // taken out of the dom flow, the element might have lost these dimensions

    element.style.height = height + 'px';
    element.style.width = width + 'px';
    var fragmentTuple = fragmentTuples.filter(function (t) {
      return t[0] === parent;
    })[0];

    if (!fragmentTuple) {
      fragmentTuple = [parent, document.createDocumentFragment()];
      fragmentTuples.push(fragmentTuple);
    }

    fragmentTuple[1].appendChild(element);
    exitingElementCount += 1;

    var stop = function stop() {
      try {
        parent.removeChild(element);
      } catch (DOMException) {// the element is already gone
      } finally {
        exitingElementCount -= 1;

        if (exitingElementCount === 0) {
          closureResolve();
        }
      }
    }; // @ts-ignore


    inProgressAnimations[id] = {
      stop: stop
    };
    return function () {
      return flipCallbacks[id].onExit(element, i, stop, decisionData);
    };
  }); // now append all the fragments from the onExit callbacks
  // (we use fragments for performance)

  fragmentTuples.forEach(function (t) {
    t[0].appendChild(t[1]);
  });

  if (!onExitCallbacks.length) {
    closureResolve();
  }

  var animateExitingElements = function animateExitingElements() {
    onExitCallbacks.forEach(function (c) {
      return c();
    });
    return promiseToReturn;
  };

  return {
    hideEnteringElements: hideEnteringElements,
    animateEnteringElements: animateEnteringElements,
    animateExitingElements: animateExitingElements
  };
};

var _default = animateUnflippedElements;
exports.default = _default;