"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getSpringConfig = exports.normalizeSpring = exports.springPresets = void 0;

var _utilities = require("../utilities");

// adapted from
// https://github.com/chenglou/react-motion/blob/master/src/presets.js
var springPresets = {
  noWobble: {
    stiffness: 200,
    damping: 26
  },
  gentle: {
    stiffness: 120,
    damping: 14
  },
  veryGentle: {
    stiffness: 130,
    damping: 17
  },
  wobbly: {
    stiffness: 180,
    damping: 12
  },
  stiff: {
    stiffness: 260,
    damping: 26
  }
};
exports.springPresets = springPresets;

function argIsSpringConfig(arg) {
  return (0, _utilities.isObject)(arg);
}

var normalizeSpring = function normalizeSpring(spring) {
  if (argIsSpringConfig(spring)) {
    return spring;
  } else if (Object.keys(springPresets).indexOf(spring) > -1) {
    return springPresets[spring];
  } else {
    return {};
  }
};

exports.normalizeSpring = normalizeSpring;

var getSpringConfig = function getSpringConfig() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      flipperSpring = _ref.flipperSpring,
      flippedSpring = _ref.flippedSpring;

  return (0, _utilities.assign)({}, springPresets.noWobble, normalizeSpring(flipperSpring), normalizeSpring(flippedSpring));
};

exports.getSpringConfig = getSpringConfig;