"use strict";

var _react = _interopRequireDefault(require("react"));

var _index = require("./index");

var _reactTestRenderer = _interopRequireDefault(require("react-test-renderer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Flipped Component', function () {
  it('adds a data-flip-id attribute', function () {
    var testRenderer = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, {
      flipId: "foo"
    }, _react.default.createElement("div", null)));

    expect(testRenderer.toJSON().props['data-flip-id']).toEqual('foo');
  });
  it('adds a data-inverse-flip-id attribute', function () {
    var testRenderer = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, {
      inverseFlipId: "foo2"
    }, _react.default.createElement("div", null)));

    expect(testRenderer.toJSON().props['data-inverse-flip-id']).toEqual('foo2');
  });
  it('only if provided with a portalKey via context, puts that on the attributes to help with document scoped selection ', function () {
    var testRenderer = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, {
      portalKey: "plants"
    }, _react.default.createElement("div", null)));

    expect(testRenderer.toJSON().props['data-portal-key']).toBe('plants');

    var testRenderer2 = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, null, _react.default.createElement("div", null)));

    expect(testRenderer2.toJSON().props['data-flip-portal-key']).toBe(undefined);
  });
  it('adds all data transform attributes if none are specified', function () {
    var testRenderer = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, {
      flipId: "foo"
    }, _react.default.createElement("div", null)));

    var flipConfig = JSON.parse(testRenderer.toJSON().props['data-flip-config']);
    expect(flipConfig).toEqual({
      translate: true,
      scale: true,
      opacity: true
    });
  });
  it("doesn't add any additional transform attributes if at least one was specified", function () {
    var testRenderer2 = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, {
      flipId: "foo",
      opacity: true
    }, _react.default.createElement("div", null)));

    var flipConfig2 = JSON.parse(testRenderer2.toJSON().props['data-flip-config']);
    expect(flipConfig2).toEqual({
      opacity: true
    });
  });
  it('adds scale transforms', function () {
    var testRenderer = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, {
      flipId: "foo",
      scale: true
    }, _react.default.createElement("div", null)));

    var flipConfig = JSON.parse(testRenderer.toJSON().props['data-flip-config']);
    expect(flipConfig).toEqual({
      scale: true
    });
  });
  it('adds translate transforms', function () {
    var testRenderer = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, {
      flipId: "foo",
      translate: true
    }, _react.default.createElement("div", null)));

    var flipConfig = JSON.parse(testRenderer.toJSON().props['data-flip-config']);
    expect(flipConfig).toEqual({
      translate: true
    });
  });
  it('passes props if it receives a function as a child', function () {
    var testRenderer = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, {
      flipId: "foo"
    }, function (flippedProps) {
      return _react.default.createElement("div", flippedProps);
    }));

    expect(testRenderer.toJSON().props).toEqual({
      'data-flip-id': 'foo',
      'data-flip-config': JSON.stringify({
        translate: true,
        scale: true,
        opacity: true
      })
    });
  });
  it('passes props if it receives a function as a child take 2', function () {
    var testRenderer = _reactTestRenderer.default.create(_react.default.createElement(_index.Flipped, {
      inverseFlipId: "bar"
    }, function (flippedProps) {
      return _react.default.createElement("div", flippedProps);
    }));

    expect(testRenderer.toJSON().props).toEqual({
      'data-inverse-flip-id': 'bar',
      'data-flip-config': JSON.stringify({
        translate: true,
        scale: true,
        opacity: true
      })
    });
  });
});