"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createStaggeredSprings = exports.normalizeSpeed = exports.createSpring = exports.createSuspendedSpring = void 0;

var _forkedRebound = require("../../../forked-rebound");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// this should get created only 1x
var springSystem = new _forkedRebound.SpringSystem();

var createSuspendedSpring = function createSuspendedSpring(flipData) {
  var _flipData$springConfi = flipData.springConfig,
      stiffness = _flipData$springConfi.stiffness,
      damping = _flipData$springConfi.damping,
      overshootClamping = _flipData$springConfi.overshootClamping,
      getOnUpdateFunc = flipData.getOnUpdateFunc,
      onAnimationEnd = flipData.onAnimationEnd,
      isGestureControlled = flipData.isGestureControlled,
      onSpringActivate = flipData.onSpringActivate;
  var spring = springSystem.createSpring(stiffness, damping);
  spring.setOvershootClampingEnabled(!!overshootClamping);

  var onSpringAtRest = function onSpringAtRest() {
    // prevent SpringSystem from caching unused springs
    spring.destroy();
    onAnimationEnd();
  };

  var springConfig = {
    onSpringActivate: onSpringActivate,
    onSpringAtRest: !isGestureControlled ? onSpringAtRest : function () {},
    onSpringUpdate: getOnUpdateFunc({
      spring: spring,
      onAnimationEnd: onAnimationEnd
    })
  };
  spring.addListener(springConfig);
  return spring;
};

exports.createSuspendedSpring = createSuspendedSpring;

var createSpring = function createSpring(flipped) {
  var spring = createSuspendedSpring(flipped);
  spring.setEndValue(1);
  return spring;
};

exports.createSpring = createSpring;

var normalizeSpeed = function normalizeSpeed(speedConfig) {
  if (typeof speedConfig !== 'number') return 1.1;
  return 1 + Math.min(Math.max(speedConfig * 5, 0), 5);
};

exports.normalizeSpeed = normalizeSpeed;

var createStaggeredSprings = function createStaggeredSprings(flippedArray) {
  var staggerConfig = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var isGestureControlled = arguments.length > 2 ? arguments[2] : undefined;

  if (!flippedArray || !flippedArray.length) {
    return;
  }

  if (staggerConfig.reverse) {
    flippedArray.reverse();
  }

  var normalizedSpeed = normalizeSpeed(staggerConfig.speed);
  var nextThreshold = 1 / Math.max(Math.min(flippedArray.length, 100), 10);
  var setEndValueFuncs = flippedArray.map(function (flipped, i) {
    var cachedGetOnUpdate = flipped.getOnUpdateFunc; // modify the update function to adjust
    // the end value of the trailing Flipped component

    flipped.getOnUpdateFunc = function (args) {
      var onUpdate = cachedGetOnUpdate(args);
      return function (spring) {
        var currentValue = spring.getCurrentValue(); // make sure trailing animations complete

        currentValue = currentValue < 0.01 ? 0 : currentValue > 0.99 ? 1 : currentValue; // direction is 0 when animation is gesture controlled and user has released before the threshold

        var updateTrailingAnimation = currentValue >= nextThreshold || isGestureControlled;

        if (updateTrailingAnimation) {
          if (setEndValueFuncs[i + 1]) {
            setEndValueFuncs[i + 1](Math.max(Math.min(currentValue * normalizedSpeed, 1), 0));
          }
        } // now call the actual update function


        onUpdate(spring);
      };
    };

    return flipped;
  }).map(function (flipped) {
    var spring = createSuspendedSpring(_objectSpread({}, flipped, {
      isGestureControlled: isGestureControlled
    }));

    if (!spring) {
      return;
    }

    return spring.setEndValue.bind(spring);
  }).filter(Boolean);

  if (setEndValueFuncs[0] && !isGestureControlled) {
    setEndValueFuncs[0](1);
  }
};

exports.createStaggeredSprings = createStaggeredSprings;