"use strict";

var _sinon = _interopRequireDefault(require("sinon"));

var _getFlippedElementPositionsBeforeUpdate = _interopRequireDefault(require("./getFlippedElementPositionsBeforeUpdate"));

var _getFlippedElementPositionsAfterUpdate = _interopRequireDefault(require("./getFlippedElementPositionsAfterUpdate"));

var _utilities = require("./utilities");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var testEl = document.querySelector('#test');
describe('getAllElements', function () {
  it('should return a func that selects all elements in the entire document with a specific data portal key if a portal key is provided', function () {
    var otherEl = document.createElement('div');
    document.querySelector('body').appendChild(otherEl);
    otherEl.innerHTML = "\n    <div>\n    <div data-flip-id=\"id-1\" data-portal-key='some-portal-key'></div>\n    <div data-flip-id=\"id-2\" data-portal-key='some-portal-key'></div>\n    </div>\n    ";
    testEl.innerHTML = "\n    <div data-flip-id=\"id-3\" data-portal-key='some-portal-key'></div>\n    <div data-flip-id=\"id-4\" data-portal-key='some-portal-key'></div>\n      </div>\n    ";
    var elements = (0, _utilities.getAllElements)(testEl, 'some-portal-key');
    expect(elements.length).to.equal(4);
  });
  it('should otherwise get all children elements with a data-flip-id attribute  ', function () {
    var otherEl = document.createElement('div');
    document.querySelector('body').appendChild(otherEl);
    otherEl.innerHTML = "\n    <div>\n    <div data-flip-id=\"id-1\" data-portal-key='some-portal-key'></div>\n    <div data-flip-id=\"id-2\" data-portal-key='some-portal-key'></div>\n    </div>\n    ";
    testEl.innerHTML = "\n    <div data-flip-id=\"id-3\" data-portal-key='some-portal-key'></div>\n    <div data-flip-id=\"id-4\" data-portal-key='some-portal-key'></div>\n      </div>\n    ";
    var elements = (0, _utilities.getAllElements)(testEl, undefined);
    expect(elements.length).to.equal(2);
  });
});
describe('getFlippedElementPositionsBeforeUpdate', function () {
  it('returns position data for elements with flip-ids', function () {
    testEl.innerHTML = "\n      <div>\n      <div data-flip-id=\"id-1\" style=\"height:100px; width: 100px; opacity: .5\"></div>\n      <div data-flip-id=\"id-2\" style=\"height:200px; width: 100px; transform: rotate(30deg)\"></div>\n      <div data-flip-id=\"id-3\" style=\"height:300px; width: 300px; transform: translate(30px)\"></div>\n      </div>\n    ";

    var _getFlippedElementPos = (0, _getFlippedElementPositionsBeforeUpdate.default)({
      element: testEl
    }),
        flippedElementPositions = _getFlippedElementPos.flippedElementPositions;

    expect(flippedElementPositions['id-1'].opacity).to.equal(0.5);
    expect(flippedElementPositions['id-1'].rect.width).to.equal(100);
    expect(flippedElementPositions['id-1'].rect.height).to.equal(100);
    expect(flippedElementPositions['id-1'].domDataForExitAnimations).to.deep.equal({});
    expect(flippedElementPositions['id-2'].domDataForExitAnimations).to.deep.equal({});
    expect(flippedElementPositions['id-2'].opacity).to.equal(1);
    expect(flippedElementPositions['id-2'].rect.width).to.equal(186.6025390625);
    expect(flippedElementPositions['id-2'].rect.height).to.equal(223.205078125);
  });
  it('for elements with onExit callbacks (and only for them), caches the DOM node and information about the parent', function () {
    testEl.innerHTML = "\n    <div id=\"parent-node-1\">\n    <div data-flip-id=\"id-2\" style=\"height:200px; width: 100px; transform: rotate(30deg)\"></div>\n    </div>\n    <div id=\"parent-node-2\">\n    <div data-flip-id=\"id-1\" style=\"height:100px; width: 100px; opacity: .5\"></div>\n    <div data-flip-id=\"id-3\" style=\"height:300px; width: 300px; transform: translate(30px)\"></div>\n    </div>\n  ";

    var _getFlippedElementPos2 = (0, _getFlippedElementPositionsBeforeUpdate.default)({
      element: testEl,
      flipCallbacks: {
        'id-2': {
          onExit: function onExit() {
            console.log('yo, i am exiting');
          }
        },
        'id-3': {
          onExit: function onExit() {
            console.log('yo, i am exiting');
          }
        }
      },
      inProgressAnimations: {}
    }),
        flippedElementPositions = _getFlippedElementPos2.flippedElementPositions;

    expect(flippedElementPositions['id-1'].domDataForExitAnimations).to.deep.equal({});
    expect(flippedElementPositions['id-2'].domDataForExitAnimations.childPosition).to.deep.equal({
      top: -11.6025390625,
      left: -43.30126953125,
      width: 186.6025390625,
      height: 223.205078125
    });
    expect(flippedElementPositions['id-2'].domDataForExitAnimations.element).to.equal(testEl.querySelector("div[data-flip-id='id-2']"));
    expect(flippedElementPositions['id-2'].domDataForExitAnimations.parent).to.equal(testEl.querySelector('#parent-node-1'));
    expect(flippedElementPositions['id-3'].domDataForExitAnimations.parent).to.deep.equal(testEl.querySelector('#parent-node-2'));
  });
  it('cancels in-progress animations and removes transforms only after recording position data, for interruptible animations', function () {
    testEl.innerHTML = "\n    <div>\n    <div data-flip-id=\"id-1\" style=\"height:100px; width: 100px; opacity: .5\"></div>\n    <div data-flip-id=\"id-2\" style=\"height:200px; width: 100px; transform: rotate(30deg)\"></div>\n    <div data-flip-id=\"id-3\" style=\"height:300px; width: 300px; transform: translate(30px)\"></div>\n    </div>\n  ";

    var fakeStop = _sinon.default.fake();

    var inProgressAnimations = {
      'id-1': {},
      'id-2': {
        spring: {
          destroy: fakeStop
        }
      }
    };
    expect(testEl.querySelector("div[data-flip-id='id-2']").style.transform).to.equal('rotate(30deg)');

    var _getFlippedElementPos3 = (0, _getFlippedElementPositionsBeforeUpdate.default)({
      element: testEl,
      flipCallbacks: {},
      inProgressAnimations: inProgressAnimations
    }),
        flippedElementPositions = _getFlippedElementPos3.flippedElementPositions;

    expect(flippedElementPositions['id-2'].rect.width).to.equal(186.6025390625);
    expect(flippedElementPositions['id-2'].rect.height).to.equal(223.205078125);
    expect(testEl.querySelector("div[data-flip-id='id-2']").style.transform).to.equal('');
    expect(fakeStop.callCount).to.equal(1);
    expect(Object.keys(inProgressAnimations).length).to.equal(0);
  });
  it('returns an ordered array of ids so that stagger effects can happen later', function () {
    testEl.innerHTML = "\n    <div>\n    <div data-flip-id=\"id-1\" style=\"height:100px; width: 100px; opacity: .5\"></div>\n    <div data-flip-id=\"id-2\" style=\"height:200px; width: 100px; transform: rotate(30deg)\"></div>\n    <div data-flip-id=\"id-3\" style=\"height:300px; width: 300px; transform: translate(30px)\"></div>\n    </div>\n  ";

    var _getFlippedElementPos4 = (0, _getFlippedElementPositionsBeforeUpdate.default)({
      element: testEl
    }),
        cachedOrderedFlipIds = _getFlippedElementPos4.cachedOrderedFlipIds;

    expect(cachedOrderedFlipIds).to.deep.eql(['id-1', 'id-2', 'id-3']);
  });
});
describe('getFlippedElementPositionsAfterUpdate', function () {
  it('returns position data for elements with flip-ids', function () {
    testEl.innerHTML = "\n    <div>\n    <div data-flip-id=\"id-1\" style=\"height:100px; width: 100px; opacity: .5\"></div>\n    <div data-flip-id=\"id-2\" style=\"height:200px; width: 100px; transform: rotate(30deg)\"></div>\n    <div data-flip-id=\"id-3\" style=\"height:300px; width: 300px; transform: translate(30px)\"></div>\n    </div>\n  ";
    var flippedElementPositions = (0, _getFlippedElementPositionsAfterUpdate.default)({
      element: testEl
    });
    expect(flippedElementPositions['id-1'].opacity).to.equal(0.5);
    expect(flippedElementPositions['id-1'].rect.width).to.equal(100);
    expect(flippedElementPositions['id-1'].rect.height).to.equal(100);
    expect(flippedElementPositions['id-1'].transform).to.equal('none');
    expect(flippedElementPositions['id-2'].opacity).to.equal(1);
    expect(flippedElementPositions['id-2'].rect.width).to.equal(186.6025390625);
    expect(flippedElementPositions['id-2'].rect.height).to.equal(223.205078125);
    expect(flippedElementPositions['id-2'].transform).to.equal('matrix(0.866025, 0.5, -0.5, 0.866025, 0, 0)');
    expect(flippedElementPositions['id-3'].opacity).to.equal(1);
    expect(flippedElementPositions['id-3'].rect.width).to.equal(300);
    expect(flippedElementPositions['id-3'].rect.height).to.equal(300);
    expect(flippedElementPositions['id-3'].transform).to.equal('matrix(1, 0, 0, 1, 30, 0)');
  });
});