"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.cancelInProgressAnimations = void 0;

var _utilities = require("../utilities");

var constants = _interopRequireWildcard(require("../../../constants"));

var _utilities2 = require("../../../utilities");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var cancelInProgressAnimations = function cancelInProgressAnimations(inProgressAnimations, animatingElements) {
  Object.keys(inProgressAnimations).forEach(function (id) {
    if (inProgressAnimations[id].spring) {
      inProgressAnimations[id].spring.destroy();
    }

    delete inProgressAnimations[id];
  });
  animatingElements.forEach(function (el) {
    el.style.transform = '';
    el.style.opacity = '';
  });
};

exports.cancelInProgressAnimations = cancelInProgressAnimations;

var getFlippedElementPositionsBeforeUpdate = function getFlippedElementPositionsBeforeUpdate(_ref) {
  var element = _ref.element,
      _ref$flipCallbacks = _ref.flipCallbacks,
      flipCallbacks = _ref$flipCallbacks === void 0 ? {} : _ref$flipCallbacks,
      _ref$inProgressAnimat = _ref.inProgressAnimations,
      inProgressAnimations = _ref$inProgressAnimat === void 0 ? {} : _ref$inProgressAnimat,
      portalKey = _ref.portalKey;
  var flippedElements = (0, _utilities.getAllElements)(element, portalKey);
  var inverseFlippedElements = (0, _utilities2.toArray)(element.querySelectorAll("[".concat(constants.DATA_INVERSE_FLIP_ID, "]")));
  var childIdsToParentBCRs = {};
  var parentBCRs = [];
  var childIdsToParents = {}; // this is for exit animations so we can re-insert exiting elements in the
  // DOM later

  flippedElements.filter(function (el) {
    return flipCallbacks && flipCallbacks[el.dataset.flipId] && flipCallbacks[el.dataset.flipId].onExit;
  }).forEach(function (el) {
    var parent = el.parentNode; // this won't work for IE11

    if (el.closest) {
      var exitContainer = el.closest("[".concat(constants.DATA_EXIT_CONTAINER, "]"));

      if (exitContainer) {
        parent = exitContainer;
      }
    }

    var bcrIndex = parentBCRs.findIndex(function (n) {
      return n[0] === parent;
    });

    if (bcrIndex === -1) {
      parentBCRs.push([parent, parent.getBoundingClientRect()]);
      bcrIndex = parentBCRs.length - 1;
    }

    childIdsToParentBCRs[el.dataset.flipId] = parentBCRs[bcrIndex][1];
    childIdsToParents[el.dataset.flipId] = parent;
  });
  var filteredFlippedElements = (0, _utilities.getRects)(flippedElements);
  var flippedElementPositions = filteredFlippedElements.map(function (_ref2) {
    var _ref3 = _slicedToArray(_ref2, 2),
        child = _ref3[0],
        childBCR = _ref3[1];

    var domDataForExitAnimations = {}; // only cache extra data for exit animations
    // if the element has an onExit listener

    if (flipCallbacks && flipCallbacks[child.dataset.flipId] && flipCallbacks[child.dataset.flipId].onExit) {
      var parentBCR = childIdsToParentBCRs[child.dataset.flipId];
      (0, _utilities2.assign)(domDataForExitAnimations, {
        element: child,
        parent: childIdsToParents[child.dataset.flipId],
        childPosition: {
          top: childBCR.top - parentBCR.top,
          left: childBCR.left - parentBCR.left,
          width: childBCR.width,
          height: childBCR.height
        }
      });
    }

    return [child.dataset.flipId, {
      rect: childBCR,
      opacity: parseFloat(window.getComputedStyle(child).opacity || '1'),
      domDataForExitAnimations: domDataForExitAnimations
    }];
  }) // @ts-ignore
  .reduce(_utilities.addTupleToObject, {}); // do this at the very end since we want to cache positions of elements
  // while they are mid-transition

  cancelInProgressAnimations(inProgressAnimations, flippedElements.concat(inverseFlippedElements));
  return {
    flippedElementPositions: flippedElementPositions,
    cachedOrderedFlipIds: filteredFlippedElements.map(function (_ref4) {
      var _ref5 = _slicedToArray(_ref4, 1),
          el = _ref5[0];

      return el.dataset.flipId;
    })
  };
};

var _default = getFlippedElementPositionsBeforeUpdate;
exports.default = _default;