"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _animateUnflippedElements = _interopRequireDefault(require("./animateUnflippedElements"));

var _animateFlippedElements = _interopRequireDefault(require("./animateFlippedElements"));

var _getFlippedElementPositionsAfterUpdate = _interopRequireDefault(require("./getFlippedElementPositions/getFlippedElementPositionsAfterUpdate"));

var constants = _interopRequireWildcard(require("../constants"));

var _utilities = require("../utilities");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createPortalScopedSelector = function createPortalScopedSelector(portalKey) {
  return function (selector) {
    return (0, _utilities.toArray)(document.querySelectorAll("[".concat(constants.DATA_PORTAL_KEY, "=\"").concat(portalKey, "\"]").concat(selector)));
  };
};

var createFlipperScopedSelector = function createFlipperScopedSelector(containerEl) {
  var tempFlipperId = Math.random().toFixed(5);
  containerEl.dataset.flipperId = tempFlipperId;
  return function (selector) {
    return (0, _utilities.toArray)(containerEl.querySelectorAll("[data-flipper-id=\"".concat(tempFlipperId, "\"] ").concat(selector)));
  };
};

var createScopedSelector = function createScopedSelector(_ref) {
  var containerEl = _ref.containerEl,
      portalKey = _ref.portalKey;

  if (portalKey) {
    return createPortalScopedSelector(portalKey);
  } else if (containerEl) {
    return createFlipperScopedSelector(containerEl);
  } else {
    return function () {
      return [];
    };
  }
};

var createGetElementFunc = function createGetElementFunc(scopedSelector) {
  return function (id) {
    return scopedSelector("[".concat(constants.DATA_FLIP_ID, "=\"").concat(id, "\"]"))[0];
  };
};

var onFlipKeyUpdate = function onFlipKeyUpdate(_ref2) {
  var _ref2$cachedOrderedFl = _ref2.cachedOrderedFlipIds,
      cachedOrderedFlipIds = _ref2$cachedOrderedFl === void 0 ? [] : _ref2$cachedOrderedFl,
      _ref2$inProgressAnima = _ref2.inProgressAnimations,
      inProgressAnimations = _ref2$inProgressAnima === void 0 ? {} : _ref2$inProgressAnima,
      _ref2$flippedElementP = _ref2.flippedElementPositionsBeforeUpdate,
      flippedElementPositionsBeforeUpdate = _ref2$flippedElementP === void 0 ? {} : _ref2$flippedElementP,
      _ref2$flipCallbacks = _ref2.flipCallbacks,
      flipCallbacks = _ref2$flipCallbacks === void 0 ? {} : _ref2$flipCallbacks,
      containerEl = _ref2.containerEl,
      applyTransformOrigin = _ref2.applyTransformOrigin,
      spring = _ref2.spring,
      debug = _ref2.debug,
      portalKey = _ref2.portalKey,
      _ref2$staggerConfig = _ref2.staggerConfig,
      staggerConfig = _ref2$staggerConfig === void 0 ? {} : _ref2$staggerConfig,
      _ref2$decisionData = _ref2.decisionData,
      decisionData = _ref2$decisionData === void 0 ? {} : _ref2$decisionData,
      handleEnterUpdateDelete = _ref2.handleEnterUpdateDelete,
      isGestureControlled = _ref2.isGestureControlled,
      onComplete = _ref2.onComplete,
      onStart = _ref2.onStart;
  var flippedElementPositionsAfterUpdate = (0, _getFlippedElementPositionsAfterUpdate.default)({
    element: containerEl,
    portalKey: portalKey
  });
  var scopedSelector = createScopedSelector({
    containerEl: containerEl,
    portalKey: portalKey
  });
  var getElement = createGetElementFunc(scopedSelector);

  var isFlipped = function isFlipped(id) {
    return flippedElementPositionsBeforeUpdate[id] && flippedElementPositionsAfterUpdate[id];
  };

  var unflippedIds = Object.keys(flippedElementPositionsBeforeUpdate).concat(Object.keys(flippedElementPositionsAfterUpdate)).filter(function (id) {
    return !isFlipped(id);
  });
  var baseArgs = {
    flipCallbacks: flipCallbacks,
    getElement: getElement,
    flippedElementPositionsBeforeUpdate: flippedElementPositionsBeforeUpdate,
    flippedElementPositionsAfterUpdate: flippedElementPositionsAfterUpdate,
    inProgressAnimations: inProgressAnimations,
    decisionData: decisionData
  };
  var animateUnFlippedElementsArgs = (0, _utilities.assign)({}, baseArgs, {
    unflippedIds: unflippedIds
  });

  var _animateUnflippedElem = (0, _animateUnflippedElements.default)(animateUnFlippedElementsArgs),
      hideEnteringElements = _animateUnflippedElem.hideEnteringElements,
      animateEnteringElements = _animateUnflippedElem.animateEnteringElements,
      animateExitingElements = _animateUnflippedElem.animateExitingElements;

  var flippedIds = cachedOrderedFlipIds.filter(isFlipped); // @ts-ignore

  var animateFlippedElementsArgs = (0, _utilities.assign)({}, baseArgs, {
    containerEl: containerEl,
    isGestureControlled: isGestureControlled,
    flippedIds: flippedIds,
    applyTransformOrigin: applyTransformOrigin,
    spring: spring,
    debug: debug,
    staggerConfig: staggerConfig,
    scopedSelector: scopedSelector,
    onComplete: onComplete
  });
  if (onStart) onStart(containerEl, decisionData); // the function handles putting flipped elements back in their original positions
  // and returns another function to actually call the flip animation

  var flip = (0, _animateFlippedElements.default)(animateFlippedElementsArgs);

  if (handleEnterUpdateDelete) {
    handleEnterUpdateDelete({
      hideEnteringElements: hideEnteringElements,
      animateEnteringElements: animateEnteringElements,
      animateExitingElements: animateExitingElements,
      animateFlippedElements: flip
    });
  } else {
    hideEnteringElements();
    animateExitingElements().then(animateEnteringElements);
    flip();
  }
};

var _default = onFlipKeyUpdate;
exports.default = _default;