"use strict";

var _index = require("./index");

describe('getSpringConfig', function () {
  it('should default to noWobble settings if nothing is provided', function () {
    expect((0, _index.getSpringConfig)()).toEqual(_index.springPresets.noWobble);
  });
  it('should allow the use of a valid spring referring to a preset ', function () {
    expect((0, _index.getSpringConfig)({
      flipperSpring: 'wobbly',
      flippedSpring: undefined
    })).toEqual(_index.springPresets.wobbly);
    expect((0, _index.getSpringConfig)({
      flippedSpring: undefined,
      flipperSpring: 'stiff'
    })).toEqual(_index.springPresets.stiff);
  });
  it('should allow the passing in of an object, and use noWobble to fill in missing keys', function () {
    expect((0, _index.getSpringConfig)({
      flippedSpring: undefined,
      flipperSpring: {
        stiffness: 400
      }
    })).toEqual({
      stiffness: 400,
      damping: 26
    });
  });
  it('should always override what is provided in flippedSpring with the data in flipperSpring if it exists', function () {
    expect((0, _index.getSpringConfig)({
      flipperSpring: {
        stiffness: 400,
        damping: 700
      },
      flippedSpring: 'stiff'
    })).toEqual(_index.springPresets.stiff);
  });
});