"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.FlippedWithContext = exports.Flipped = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _FlipToolkit = require("../FlipToolkit");

var _context = require("../Flipper/context");

var _utilities = require("../FlipToolkit/utilities");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function isFunction(child) {
  return typeof child === 'function';
} // This wrapper creates child components for the main Flipper component


var Flipped = function Flipped(_ref) {
  var children = _ref.children,
      flipId = _ref.flipId,
      inverseFlipId = _ref.inverseFlipId,
      portalKey = _ref.portalKey,
      gestureHandlers = _ref.gestureHandlers,
      rest = _objectWithoutProperties(_ref, ["children", "flipId", "inverseFlipId", "portalKey", "gestureHandlers"]);

  var child = children;
  var isFunctionAsChildren = isFunction(child);

  if (!isFunctionAsChildren) {
    try {
      child = _react.Children.only(children);
    } catch (e) {
      throw new Error('Each Flipped component must wrap a single child');
    }
  } // if nothing is being animated, assume everything is being animated


  if (!rest.scale && !rest.translate && !rest.opacity) {
    _FlipToolkit.utilities.assign(rest, {
      translate: true,
      scale: true,
      opacity: true
    });
  }

  var dataAttributes = _defineProperty({}, _FlipToolkit.constants.DATA_FLIP_CONFIG, JSON.stringify(rest));

  if (flipId) dataAttributes[_FlipToolkit.constants.DATA_FLIP_ID] = String(flipId);else if (inverseFlipId) dataAttributes[_FlipToolkit.constants.DATA_INVERSE_FLIP_ID] = String(inverseFlipId);

  if (portalKey) {
    dataAttributes[_FlipToolkit.constants.DATA_PORTAL_KEY] = portalKey;
  }

  if (isFunctionAsChildren) {
    return child(dataAttributes);
  }

  if (gestureHandlers) {
    (0, _utilities.assign)(dataAttributes, gestureHandlers);
  }

  return (0, _react.cloneElement)(child, dataAttributes);
};

exports.Flipped = Flipped;

var FlippedWithContext = function FlippedWithContext(_ref2) {
  var children = _ref2.children,
      flipId = _ref2.flipId,
      shouldFlip = _ref2.shouldFlip,
      shouldInvert = _ref2.shouldInvert,
      onAppear = _ref2.onAppear,
      onStart = _ref2.onStart,
      onStartImmediate = _ref2.onStartImmediate,
      onComplete = _ref2.onComplete,
      onExit = _ref2.onExit,
      onSpringUpdate = _ref2.onSpringUpdate,
      gestureHandlers = _ref2.gestureHandlers,
      rest = _objectWithoutProperties(_ref2, ["children", "flipId", "shouldFlip", "shouldInvert", "onAppear", "onStart", "onStartImmediate", "onComplete", "onExit", "onSpringUpdate", "gestureHandlers"]);

  if (!children) {
    return null;
  }

  if (rest.inverseFlipId) {
    return _react.default.createElement(Flipped, rest, children);
  }

  return _react.default.createElement(_context.PortalContext.Consumer, null, function (portalKey) {
    return _react.default.createElement(_context.FlipContext.Consumer, null, function (data) {
      // if there is no surrounding Flipper component,
      // we don't want to throw an error, so check
      // that data exists and is not the default string
      if (_FlipToolkit.utilities.isObject(data) && flipId) {
        data[flipId] = {
          shouldFlip: shouldFlip,
          shouldInvert: shouldInvert,
          onAppear: onAppear,
          onStart: onStart,
          onStartImmediate: onStartImmediate,
          onComplete: onComplete,
          onExit: onExit,
          onSpringUpdate: onSpringUpdate
        };
      }

      return _react.default.createElement(Flipped, _extends({
        flipId: flipId
      }, rest, {
        portalKey: portalKey,
        gestureHandlers: gestureHandlers
      }), children);
    });
  });
}; // @ts-ignore


exports.FlippedWithContext = FlippedWithContext;

if (false) {
  FlippedWithContext.propTypes = {
    children: _propTypes.default.oneOfType([_propTypes.default.node, _propTypes.default.func]).isRequired,
    inverseFlipId: _propTypes.default.string,
    flipId: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.number]),
    opacity: _propTypes.default.bool,
    translate: _propTypes.default.bool,
    scale: _propTypes.default.bool,
    transformOrigin: _propTypes.default.string,
    spring: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.object]),
    onStart: _propTypes.default.func,
    onStartImmediate: _propTypes.default.func,
    onComplete: _propTypes.default.func,
    onAppear: _propTypes.default.func,
    onSpringUpdate: _propTypes.default.func,
    shouldFlip: _propTypes.default.func,
    shouldInvert: _propTypes.default.func,
    onExit: _propTypes.default.func,
    portalKey: _propTypes.default.string,
    stagger: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.bool])
  };
}

FlippedWithContext.displayName = 'Flipped';
var _default = FlippedWithContext;
exports.default = _default;